'------------------------- SPI XMM Support Routines -------------------------
'
CON


VAR
  long  command
  long  hubaddrs
  long  ramaddrs
  long  blocklen
  long  errx
  long  cog

  
PUB start : err_
' Initialise the SPI Ram driver. 
  command := "I"
  cog := 1 + cognew(@entry, @command)
  if cog == 0
    err_ := $FF                 ' error = no cog
  else
    repeat while command        ' driver cog sets =0 when done
    err_ := errx                ' driver cog sets =0 if no error, else xx = error code


PUB DoCmd(command_, hub_address, ram_address, block_length) : err_
' valid commands:
'   R => Read
'   W => Write
'   X => Tri-state SRAM

  hubaddrs := hub_address       ' hub address start
  ramaddrs := ram_address       ' ram address start
  blocklen := block_length      ' block length
  command  := command_          ' must be last !!
' Wait for command to complete and get status
  repeat while command          ' driver cog sets =0 when done
  err_ := errx                  ' driver cog sets =0 if no error, else xx = error code

PUB stop
   if cog
      cogstop(cog~ - 1)  

'
DAT
                        ORG       0

entry
                                      '  +-- These instructions are overwritten as variables after start
comptr                  mov     comptr, par     ' -|  hub pointer to command                
hubptr                  mov     hubptr, par     '  |  hub pointer to hub address            
ramptr                  add     hubptr, #4      '  |  hub pointer to ram address            
lenptr                  mov     ramptr, par     '  |  hub pointer to length                 
errptr                  add     ramptr, #8      '  |  hub pointer to error status           
cmd                     mov     lenptr, par     '  |  command  R/W/B/S                  
hubaddr                 add     lenptr, #12     '  |  hub address                           
ramaddr                 mov     errptr, par     '  |  ram address                           
len                     add     errptr, #16     '  |  length                                
err                     nop                     ' -+  error status returned (=0=false=good) 

                        'call    #XMM_Activate

done                    wrlong  err, errptr             ' status  =0=false=good, else error x
                        wrlong  zero, comptr            ' command =0 (done)

' wait for a command
cmd_loop
                        rdlong  cmd, comptr     wz      ' command ?
              if_z      jmp     #cmd_loop

' decode command
                        cmp     cmd, #"R"       wz      ' R = read block
              if_z      jmp     #rdblock
              
                        cmp     cmd, #"W"       wz      ' W = write block
              if_z      jmp     #wrblock
              
                        cmp     cmd, #"X"       wz      ' X = set tristate
              if_z      jmp     #XMM_Tristate

                        mov     err, cmd                ' error = cmd (unknown command)
                        jmp     #done


rdblock                 call    #ram_open
                        
rdloop                  call    #XMM_InByte             '  read byte from address into data_8
                        wrbyte  data_8, hubaddr          ' write data_8 to hubaddr ie copy byte to hub
                        add     hubaddr, #1              ' add 1 to hub address
                        add     ramaddr, #1              ' add 1 to hub address
                        djnz    len, #rdloop             ' loop until done
                        jmp     #done
                        
wrblock                 call    #ram_open
                        
wrloop                  rdbyte  data_8, hubaddr          ' copy byte from hub
                        call    #XMM_OutByte             ' write byte from data_8 to address
                        add     hubaddr, #1              ' add 1 to hub address
                        add     ramaddr, #1              ' add 1 to hub address 
                        djnz    len, #wrloop             ' loop until done
                        jmp     #done

ram_open                rdlong  hubaddr, hubptr         ' get hub address
                        rdlong  ramaddr, ramptr         ' get ram address
                        rdlong  len, lenptr             ' get length
ram_open_ret            ret                        

'
XMM_Tristate
                        mov     dira, #0                  ' all inputs
                        mov     outa, #0                  ' and =0
                        jmp     #done 
'
'
' set external RAM address
XMM_SetAddr
                        ' ADR HI
                        mov     _adr, ramaddr             ' work address        
                        and     _adr, _m_A18_A11          ' hi part
                        shr     _adr, #3                  ' move to latch port
                        or      _adr, _BUS_AL_HI          ' BUS + AL hi
                        mov     outa, _adr                ' BUS + AL hi + ADR
                        and     _adr, _BUS_AL_LO
                        mov     outa, _adr                ' BUS + AL lo + LATCH
                        
                        ' ADR LO
                        mov     _adr, ramaddr             ' work address 
                        and     _adr, _m_A10_A00          ' lo part
                        shl     _adr, #8                  ' move to address port
                        'mov     outa, _adr                ' /RAMx + A10-A0
XMM_SetAddr_ret
                        ret              
'
'
XMM_OutByte     
                        mov     outa, _BUS_INIT            ' all de-selected
                        mov     dira, _DIR_OUT             ' D7..D0 as output
                        call    #XMM_SetAddr

                        and     data_8, #$FF               ' only D7-D0
                        or      _adr, data_8
                        mov     outa, _adr                 ' set  data
                        andn    outa, _BUS_WR_R1            ' /WR = 0 /RAMCS1 = 0              
                        nop
                        or      outa, _BUS_WR_R1            ' /WR = 1 /RAMCS1 = 1
XMM_OutByte_ret
                        ret              
'
'
XMM_InByte
                        mov     outa, _BUS_INIT            ' all de-selected
                        mov     dira, _DIR_IN              ' D7..D0 as input
                        call    #XMM_SetAddr

                        or      _adr,_BUS_RD_R1             ' address <= $07FFFF
                        mov     outa,_adr                   ' /RAMx + A10-A0
                        nop                                 ' SRAM delay
                        'nop
                        mov     data_8, ina
                        or      outa, _BUS_WR_R1            ' /WR = 1 /RAMCS1 = 1
                        and     data_8, #$FF                ' only D7-D0                
XMM_InByte_ret
                        ret

zero                    long    0
data_8                  long    0
_adr                    long    0
_tmp                    long    0 

'
'     
 '                               RT SS H W B H A P P R R AAA AAAAAAAA DDDDDDDD
 '                               XX DC S R U B L R R A A 198 76543210 76543210
 '                                  AL     S E   O O M M 0
 '                                         C A   P P C C 
 '                                         L T   2 1 S S 
 '                                         K         2 1
_DIR_OUT                long    %00_00_0_1_0_0_1_0_0_1_1_111_11111111_11111111
_DIR_IN                 long    %00_00_0_1_0_0_1_0_0_1_1_111_11111111_00000000
_BUS_INIT               long    %00_00_0_1_0_0_0_0_0_1_1_000_00000000_00000000  
_m_A18_A11              long    %00_00_0_0_0_0_0_0_0_0_0_111_11111000_00000000
_m_A10_A00              long    %00_00_0_0_0_0_0_0_0_0_0_000_00000111_11111111
_BUS_AL_HI              long    %00_00_0_1_0_0_1_0_0_1_1_000_00000000_00000000
_BUS_AL_LO              long    %00_00_0_1_0_0_0_0_0_1_1_111_11111111_00000000
_BUS_RD_R1              long    %00_00_0_1_0_0_0_0_0_1_0_000_00000000_00000000
'_BUS_WR_R1              long    %00_00_0_0_0_0_0_0_0_1_0_111_11111111_11111111
_BUS_WR_R1              long    %00_00_0_1_0_0_0_0_0_0_1_000_00000000_00000000

 

'
'---------------------- End of SRAM XMM Support Routines ---------------------
'