' Speed-Test:   (1000 / time in ms)
'
' 07.04.2015    5348            1,86986 MHz
'
' 09.04.2012 qz80.spin:   included ram_address routine into rd_byte & wr_byte
'               5192            1,92604 MHz   (+3%)
'


CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  HBEAT_LED = 24

  'The following taken from SIMH Altair simulator CBIOSX.MAC
  ndisks  = 8                     ' total number of Altair disks
  tracks  = 254                   ' number of tracks for regular drives
  track1  = tracks+1              ' indicator for unknown track position
  asecsiz = 137                   ' sector size Altair
  csecsiz = $0080                 ' sector size CP/M

  'I/O control block commands
  io_cmd_out   = $01 
  io_cmd_in    = $02 
  io_cmd_break = $03   

  selout = 8                      ' port to select and enable controller and drive (OUT)
  statin = 8                      ' port indicating status of drive and controller (IN)
  dskcon = 9                      ' port to control disc function (OUT)
  secpos = 9                      ' port to indicate current sector position of disk (IN)
  dskwrit = 10                    ' port to write data (OUT)
  dskread = 10                    ' port to read data (IN)

  constat = 16                    ' sio port 1 status port
  condata = 17                    ' sio port 1 data port
  punstat = 18                    ' sio port 2 status port
  pundata = 19                    ' sio port 2 data port

  ledport  = $00

  hdskport = $fd                    ' control port for simulated hard disk
  simhport = $FE
  srport   = $FF                  ' Altair front panel switch register port

  ' commands for disk controller (dskcon)
  cstepin = $01                   ' step in command
  cstepot = $02                   ' step out command
  cload   = $04                   ' load head to drive surface command
  cuload  = $08                   ' unload head from drive surface command
  cwrseq  = $80                   ' 'start write enable sequence' command
   
  SIMHgetversion      = 6
  SIMHreset           = 14
  SIMHhasbankedmemory = 18
  SIMHstoptimer       = 22
  SIMHresettimer      = 25
  SIMHreadtimer       = 26
  SIMHsleep           = 27

VAR
  long  params[3]
  long  io_command
  long  irq
  'long  op_stats[256]

  byte  altair_drive        'The Altair disk drive currently selected, -1 = none
  byte  altair_track[16]    'The current track position on each Altair disk drive
  byte  altair_sector       'The current sector on an altair disk
  byte  altair_bidx         'Byte index into Altair disk sectors
  byte  altair_write_enable 'Bytes can be written to the Altair disk when = 1

  byte  disk_buff[asecsiz]

  long  x
  long  key
  long  switch_reg

  long  hub_addr

  byte  io_cmd
  byte  io_port
  byte  io_data

  byte  SIMHcmd
  byte  simh_cmd_cnt
  long  simh_cmd_ptr

  long	cog
  long  io_stack[64]

  long  tregs                                                   ' address of timer regs
  word  timer_ms

  long  time

OBJ

  cpu     : "qz80"
  mem     : "altair_mem"
  disk    : "sdcard-emu"
  kb      : "PC_Keyboard"
  TV      : "PC_Text"
  timer   : "jm_etimer"


PUB start

  stop

  switch_reg := $08                       '$08 is the switch setting SIMH recommends for 4k Basic etc to run

  'Start the terminal
  TV.start
  TV.out(0)
  TV.str(string("Propaltair qz80-0.9.32 (CP/M 2.2)",13))

  'Start the keyboard

  TV.str(string("Starting keyboard ... "))
  x := \kb.start    
  if x == 0
    TV.str(string("FAIL",13))
  else
    TV.str(string("OK",13))
 
  'Start the disk
  TV.str(string("Starting disks (A:B:C:D:E:F:G:H:) ... "))
  x := \disk.start
  if x <> 0
    TV.str(string("FAIL",13))
  else
    TV.str(string("OK",13))


  TV.str(string("Loading SRAM ... "))
  'Copy  CBIOSX.COM, CCP.COM and BDOS.COM to SRAM
  mem.LoadSRAM   
  TV.str(string("done", 13)) 

  'Start timer
  TV.str(string("Starting Timer ..."))
  x := \timer.start(HBEAT_LED, timer#ET_HOLD)
  if x == 0
    TV.str(string("FAIL",13))
  else
    TV.str(string("OK",13))
  tregs := timer.address                                           ' get address of registers


  'longfill(@op_stats, 0, 256)

  'Start the CPU
  TV.str(string("Starting CPU ..."))
  params[0] := @io_command
  params[1] := @irq
  'params[2] := @op_stats

  x := \cpu.start(@params, 0)
  if x == 0
    TV.str(string("FAIL",13))
  else
    TV.str(string("OK",13))

  TV.str(string("Starting I/O ..."))
  x := cog := COGNEW(io_handler, @io_stack) + 1
  if x == 0
    TV.str(string("FAIL",13))
  else
    TV.str(string("OK",13))
    TV.str(string("Initialization complete. "))
    TV.dec(free_cogs)
    TV.str(string(" cogs free.",13))

  return

PUB stop
'' Stop CPU simulator - frees a cog
  if cog
    cogstop(cog~ - 1)

'***************************************************************************************************************
' I/O command handler
'
PRI io_handler
  repeat
    repeat while (io_command & $FF000000) == 0

    io_cmd  := io_command >> 24
    io_port := (io_command >> 8)

    case io_cmd
      io_cmd_in:
        io_input
        io_command := (io_command & !$FF) | io_data     'Return data and set flag ready for next io

      io_cmd_out:
        io_output

      io_cmd_break:
        io_break

    io_command &= $00FFFFFF                  'Set flag ready for next io


'***************************************************************************************************************
' Input command handler
'
PRI io_input
  case io_port
    srport:                                  'Altair front panel switches register
      io_data := switch_reg

    simhport:                                'SIMH control port
      case SIMHcmd
        SIMHhasbankedmemory:
          io_data := 0
          SIMHcmd := 0

        SIMHgetversion:
          if simh_cmd_cnt > 0
            io_data := byte[simh_cmd_ptr]
            simh_cmd_cnt--
            simh_cmd_ptr++
          else
            SIMHcmd := 0

        SIMHreadtimer:
          if simh_cmd_cnt == 2
            io_data := timer_ms
            simh_cmd_cnt := 1
          elseif simh_cmd_cnt == 1
            io_data := timer_ms >> 8
            simh_cmd_cnt := 0
            SIMHcmd := 0

        other:
          SIMHcmd := 0

    constat:                                 'Console status port.
      if kb.gotkey == 0                      'Is there a key press ?
        io_data := %00000010                 'No: Return no char ready for input and output rdy.
      else
        key := kb.key                        'Yes: Get the key
        if key <> 0                          'NULL character ?
          io_data := %00000011               'Return char ready for input and output rdy.
        else
          io_data := %00000010               'Yes: Return no char ready for input and output rdy.

    condata:                                 'Console data port ?
      io_data := key                         'Return the keyboard character

    statin:                                  'Altair disk status port
      io_data := 0
      if altair_track[altair_drive] <> 0     'Now on track zero ?
        io_data |= %01000000

    secpos:                                  'Altair disk sector position port
      altair_sector := altair_sector + 1
      if altair_sector > 31
        altair_sector := 0
      io_data := (altair_sector << 1)        'Return sector and positioned bit
      altair_bidx := 0

    dskread:                                 'Altair disk read port
      if altair_bidx == 0
        'Read a new block from SD card
        x := \disk.readblock(altair_drive, altair_track[altair_drive], altair_sector, @disk_buff)
        'time := -cnt
      io_data := disk_buff[altair_bidx]
      'if altair_bidx < asecsiz
      altair_bidx++
{{
      if altair_bidx == asecsiz
        time += cnt - 544
        TV.str(string("Readblock: "))
        TV.dec(time)
        TV.out(13)
}}
    hdskport:

    other:
      io_data := io_port

'***************************************************************************************************************
' Output command handler
'
PRI io_output

  io_data    := io_command & $FF

  case io_port
    ledport:
      TV.str(string("LED port = "))
      TV.hex(io_data,2)
      TV.out(13)

    simhport:
      SIMHcmd := io_data
      case SIMHcmd                                'SIMH control port

        SIMHgetversion:
          simh_cmd_cnt := @simh_version_len - @simh_version
          simh_cmd_ptr := @simh_version

        SIMHsleep:
          SIMHcmd := 0

        SIMHresettimer:
          timer.reset
          timer.resume

        SIMHreadtimer:
          timer.hold
          timer_ms := long[tregs][0] + long[tregs][1] * 1000
          simh_cmd_cnt := 2

        SIMHreset:
          SIMHcmd := 0

        SIMHhasbankedmemory:

        SIMHstoptimer:
          SIMHcmd := 0

        other:
          TV.str(string("SIMH output: "))
          TV.hex(SIMHcmd,2)
          TV.out(13)

    condata:                                 'Console data port
      if io_data <> 10
{{
        if io_data == ">"
          TV.out(13)
          repeat x from 0 to 255
            if op_stats[x] > 0
              TV.hex(x,2)
              TV.out(32)
              TV.hex(op_stats[x], 8)
              TV.out(13)
          repeat
          'TV.str(string("stop", 13))
}}
        TV.out(io_data)

    constat:

    punstat:

    selout:                                   'Altair disk controller select port
      if io_data & %10000000
        altair_drive := -1
      else
        altair_drive := io_data & %00001111

    dskcon:                                      'Altair disk function port
      if (io_data & cstepin) > 0                 'Step in command ?
        if altair_track[altair_drive] < tracks
          altair_track[altair_drive]++
      if (io_data & cstepot) > 0                 'Step out command ?
        if altair_track[altair_drive] > 0
          altair_track[altair_drive]--

      if (io_data & cload) > 0                   'Load head to drive surface command ?
      if (io_data & cuload) > 0                  'Unload head from drive surface command ?
      if (io_data & cwrseq) > 0                  ' 'start write enable sequence' command
        altair_bidx := 0
        altair_write_enable := 1

    dskwrit:                                  'Altair disk write port
      if altair_bidx < asecsiz
        disk_buff[altair_bidx++] := io_data
        if altair_bidx == asecsiz
          'Write the buffer to the SD card
          x := \disk.writeblock(altair_drive, altair_track[altair_drive], altair_sector, @disk_buff)
          altair_write_enable := 0

    hdskport:

    other:

'***************************************************************************************************************
' Break
'
PRI io_break

  hub_addr := io_command >> 8                ' get gegisters HUB address

  TV.str(string("BC:"))
  TV.hex(word[hub_addr][0], 4)
  TV.str(string(" DE:"))
  TV.hex(word[hub_addr][1], 4)
  TV.str(string(" HL:"))
  TV.hex(word[hub_addr][2], 4)
  TV.str(string(" AF:"))
  TV.hex(word[hub_addr][3], 4)
  TV.str(string(" BC'"))
  TV.hex(word[hub_addr][4], 4)
  TV.str(string(" DE'"))
  TV.hex(word[hub_addr][5], 4)
  TV.str(string(" HL'"))
  TV.hex(word[hub_addr][6], 4)
  TV.str(string(" AF'"))
  TV.hex(word[hub_addr][7], 4)
  TV.out(13)
  TV.str(string("R:"))
  TV.hex((byte[hub_addr][16] & $7f) | byte[hub_addr][17],4)
  TV.str(string(" IFF1:"))
  TV.hex(byte[hub_addr][18] & 1, 1)
  TV.str(string(" IFF2:"))
  TV.hex((byte[hub_addr][18] >> 1) & 1, 1)
  TV.str(string(" IM:"))
  TV.hex((byte[hub_addr][18] >> 2) & 3, 1)
  TV.str(string(" IX:"))
  TV.hex(word[hub_addr][10],4)
  TV.str(string(" IY:"))
  TV.hex(word[hub_addr][11],4)
  TV.str(string(" SP:"))
  TV.hex(word[hub_addr][12],4)
  TV.str(string(" PC:"))
  TV.hex(word[hub_addr][13],4)
  TV.str(string(" OP:"))                             'WW
  TV.hex(byte[hub_addr][14],4)                          'WW
  TV.out(13)
  key := kb.key
  repeat while key == 0
    key := kb.key

PUB free_cogs : free | i, cogs[8], jmp_0
  jmp_0 := %010111_0001_1111_000000000_000000000
  repeat while (cogs[free] := cognew(@jmp_0, 0)) => 0
    free++
  if free
    repeat i from 0 to free - 1
      cogstop(cogs[i])
  return free

DAT
simh_version            byte "SIMH004", 0
simh_version_len

{{
'************************************************************************************************************
/*  Z80 or 8080 programs communicate with the SIMH pseudo device via port 0xfe.
        The following principles apply:

    1)  For commands that do not require parameters and do not return results
        ld  a,<cmd>
        out (0feh),a
        Special case is the reset command which needs to be send 128 times to make
        sure that the internal state is properly reset.

    2)  For commands that require parameters and do not return results
        ld  a,<cmd>
        out (0feh),a
        ld  a,<p1>
        out (0feh),a
        ld  a,<p2>
        out (0feh),a
        ...
        Note: The calling program must send all parameter bytes. Otherwise
        the pseudo device is left in an undefined state.

    3)  For commands that do not require parameters and return results
        ld  a,<cmd>
        out (0feh),a
        in  a,(0feh)    ; <A> contains first byte of result
        in  a,(0feh)    ; <A> contains second byte of result
        ...
        Note: The calling program must request all bytes of the result. Otherwise
        the pseudo device is left in an undefined state.

    4)  For commands that do require parameters and return results
        ld  a,<cmd>
        out (0feh),a
        ld  a,<p1>
        out (0feh),a
        ld  a,<p2>
        out (0feh),a
        ...             ; send all parameters
        in  a,(0feh)    ; <A> contains first byte of result
        in  a,(0feh)    ; <A> contains second byte of result
        ...

*/

enum simhPseudoDeviceCommands { /* do not change order or remove commands, add only at the end          */
    printTimeCmd,               /*  0 print the current time in milliseconds                            */
    startTimerCmd,              /*  1 start a new timer on the top of the timer stack                   */
    stopTimerCmd,               /*  2 stop timer on top of timer stack and show time difference         */
    resetPTRCmd,                /*  3 reset the PTR device                                              */
    attachPTRCmd,               /*  4 attach the PTR device                                             */
    detachPTRCmd,               /*  5 detach the PTR device                                             */
    getSIMHVersionCmd,          /*  6 get the current version of the SIMH pseudo device                 */
    getClockZSDOSCmd,           /*  7 get the current time in ZSDOS format                              */
    setClockZSDOSCmd,           /*  8 set the current time in ZSDOS format                              */
    getClockCPM3Cmd,            /*  9 get the current time in CP/M 3 format                             */
    setClockCPM3Cmd,            /* 10 set the current time in CP/M 3 format                             */
    getBankSelectCmd,           /* 11 get the selected bank                                             */
    setBankSelectCmd,           /* 12 set the selected bank                                             */
    getCommonCmd,               /* 13 get the base address of the common memory segment                 */
    resetSIMHInterfaceCmd,      /* 14 reset the SIMH pseudo device                                      */
    showTimerCmd,               /* 15 show time difference to timer on top of stack                     */
    attachPTPCmd,               /* 16 attach PTP to the file with name at beginning of CP/M command line*/
    detachPTPCmd,               /* 17 detach PTP                                                        */
    hasBankedMemoryCmd,         /* 18 determines whether machine has banked memory                      */
    setZ80CPUCmd,               /* 19 set the CPU to a Z80                                              */
    set8080CPUCmd,              /* 20 set the CPU to an 8080                                            */
    startTimerInterruptsCmd,    /* 21 start timer interrupts                                            */
    stopTimerInterruptsCmd,     /* 22 stop timer interrupts                                             */
    setTimerDeltaCmd,           /* 23 set the timer interval in which interrupts occur                  */
    setTimerInterruptAdrCmd,    /* 24 set the address to call by timer interrupts                       */
    resetStopWatchCmd,          /* 25 reset the millisecond stop watch                                  */
    readStopWatchCmd,           /* 26 read the millisecond stop watch                                   */
    SIMHSleepCmd,               /* 27 let SIMH sleep for SIMHSleep microseconds                         */
    getHostOSPathSeparatorCmd,  /* 28 obtain the file path separator of the OS under which SIMH runs    */
    getHostFilenamesCmd,        /* 29 perform wildcard expansion and obtain list of file names          */
    readURLCmd,                 /* 30 read the contents of an URL                                       */
    getCPUClockFrequency,       /* 31 get the clock frequency of the CPU                                */
    setCPUClockFrequency,       /* 32 set the clock frequency of the CPU                                */
    kSimhPseudoDeviceCommands
    }
}}

